@@ -1,32 +1,43 @@
 """Checker Manager and Checker classes."""
-import collections
+from __future__ import annotations
+
+import argparse
+import contextlib
 import errno
 import logging
+import multiprocessing.pool
+import operator
 import signal
-import sys
 import tokenize
-from typing import Dict, List, Optional, Tuple
-
-try:
-    import multiprocessing
-except ImportError:
-    multiprocessing = None  # type: ignore
+from typing import Any
+from typing import Generator
+from typing import List
+from typing import Optional
+from typing import Sequence
+from typing import Tuple
 
 from flake8 import defaults
 from flake8 import exceptions
 from flake8 import processor
 from flake8 import utils
+from flake8.discover_files import expand_paths
+from flake8.options.parse_args import parse_args
+from flake8.plugins.finder import Checkers
+from flake8.plugins.finder import LoadedPlugin
+from flake8.style_guide import StyleGuideManager
+
+Results = List[Tuple[str, int, int, str, Optional[str]]]
 
 LOG = logging.getLogger(__name__)
 
 SERIAL_RETRY_ERRNOS = {
     # ENOSPC: Added by sigmavirus24
     # > On some operating systems (OSX), multiprocessing may cause an
-    # > ENOSPC error while trying to trying to create a Semaphore.
+    # > ENOSPC error while trying to create a Semaphore.
     # > In those cases, we should replace the customized Queue Report
     # > class with pep8's StandardReport class to ensure users don't run
     # > into this problem.
-    # > (See also: https://gitlab.com/pycqa/flake8/issues/74)
+    # > (See also: https://github.com/pycqa/flake8/issues/117)
     errno.ENOSPC,
     # NOTE(sigmavirus24): When adding to this list, include the reasoning
     # on the lines before the error code and always append your error
@@ -34,20 +45,43 @@ SERIAL_RETRY_ERRNOS = {
     # noise in diffs.
 }
 
-
-def _multiprocessing_is_fork():  # type () -> bool
-    """Class state is only preserved when using the `fork` strategy."""
-    if sys.version_info >= (3, 4):
-        return (
-            multiprocessing
-            # https://github.com/python/typeshed/pull/3415
-            and multiprocessing.get_start_method() == "fork"  # type: ignore
-        )
-    else:
-        return multiprocessing and not utils.is_windows()
+_mp_plugins: Checkers
+_mp_options: argparse.Namespace
 
 
-class Manager(object):
+@contextlib.contextmanager
+def _mp_prefork(
+    plugins: Checkers, options: argparse.Namespace
+) -> Generator[None, None, None]:
+    # we can save significant startup work w/ `fork` multiprocessing
+    global _mp_plugins, _mp_options
+    _mp_plugins, _mp_options = plugins, options
+    try:
+        yield
+    finally:
+        del _mp_plugins, _mp_options
+
+
+def _mp_init(argv: Sequence[str]) -> None:
+    global _mp_plugins, _mp_options
+
+    # Ensure correct signaling of ^C using multiprocessing.Pool.
+    signal.signal(signal.SIGINT, signal.SIG_IGN)
+
+    try:
+        _mp_plugins, _mp_options  # for `fork` this'll already be set
+    except NameError:
+        plugins, options = parse_args(argv)
+        _mp_plugins, _mp_options = plugins.checkers, options
+
+
+def _mp_run(filename: str) -> tuple[str, Results, dict[str, int]]:
+    return FileChecker(
+        filename=filename, plugins=_mp_plugins, options=_mp_options
+    ).run_checks()
+
+
+class Manager:
     """Manage the parallelism and checker instances for each plugin and file.
 
     This class will be responsible for the following:
@@ -66,85 +100,54 @@ class Manager(object):
       together and make our output deterministic.
     """
 
-    def __init__(self, style_guide, arguments, checker_plugins):
-        """Initialize our Manager instance.
-
-        :param style_guide:
-            The instantiated style guide for this instance of Flake8.
-        :type style_guide:
-            flake8.style_guide.StyleGuide
-        :param list arguments:
-            The extra arguments parsed from the CLI (if any)
-        :param checker_plugins:
-            The plugins representing checks parsed from entry-points.
-        :type checker_plugins:
-            flake8.plugins.manager.Checkers
-        """
-        self.arguments = arguments
+    def __init__(
+        self,
+        style_guide: StyleGuideManager,
+        plugins: Checkers,
+        argv: Sequence[str],
+    ) -> None:
+        """Initialize our Manager instance."""
         self.style_guide = style_guide
         self.options = style_guide.options
-        self.checks = checker_plugins
+        self.plugins = plugins
         self.jobs = self._job_count()
-        self.checkers = []  # type: List[FileChecker]
         self.statistics = {
             "files": 0,
             "logical lines": 0,
             "physical lines": 0,
             "tokens": 0,
         }
+        self.exclude = (*self.options.exclude, *self.options.extend_exclude)
+        self.argv = argv
+        self.results: list[tuple[str, Results, dict[str, int]]] = []
 
-    def _process_statistics(self):
-        for checker in self.checkers:
+    def _process_statistics(self) -> None:
+        for _, _, statistics in self.results:
             for statistic in defaults.STATISTIC_NAMES:
-                self.statistics[statistic] += checker.statistics[statistic]
-        self.statistics["files"] += len(self.checkers)
+                self.statistics[statistic] += statistics[statistic]
+        self.statistics["files"] += len(self.filenames)
 
-    def _job_count(self):
-        # type: () -> int
+    def _job_count(self) -> int:
         # First we walk through all of our error cases:
         # - multiprocessing library is not present
-        # - we're running on windows in which case we know we have significant
-        #   implemenation issues
         # - the user provided stdin and that's not something we can handle
         #   well
-        # - we're processing a diff, which again does not work well with
-        #   multiprocessing and which really shouldn't require multiprocessing
         # - the user provided some awful input
-        if not _multiprocessing_is_fork():
-            LOG.warning(
-                "The multiprocessing module is not available. "
-                "Ignoring --jobs arguments."
-            )
-            return 0
 
-        if utils.is_using_stdin(self.arguments):
+        if utils.is_using_stdin(self.options.filenames):
             LOG.warning(
                 "The --jobs option is not compatible with supplying "
                 "input using - . Ignoring --jobs arguments."
             )
             return 0
 
-        if self.options.diff:
-            LOG.warning(
-                "The --diff option was specified with --jobs but "
-                "they are not compatible. Ignoring --jobs arguments."
-            )
-            return 0
-
         jobs = self.options.jobs
-        if jobs != "auto" and not jobs.isdigit():
-            LOG.warning(
-                '"%s" is not a valid parameter to --jobs. Must be one '
-                'of "auto" or a numerical value, e.g., 4.',
-                jobs,
-            )
-            return 0
 
         # If the value is "auto", we want to let the multiprocessing library
         # decide the number based on the number of CPUs. However, if that
         # function is not implemented for this particular value of Python we
         # default to 1
-        if jobs == "auto":
+        if jobs.is_auto:
             try:
                 return multiprocessing.cpu_count()
             except NotImplementedError:
@@ -152,9 +155,9 @@ class Manager(object):
 
         # Otherwise, we know jobs should be an integer and we can just convert
         # it to an integer
-        return int(jobs)
+        return jobs.n_jobs
 
-    def _handle_results(self, filename, results):
+    def _handle_results(self, filename: str, results: Results) -> int:
         style_guide = self.style_guide
         reported_results_count = 0
         for (error_code, line_number, column, text, physical_line) in results:
@@ -168,83 +171,7 @@ class Manager(object):
             )
         return reported_results_count
 
-    def is_path_excluded(self, path):
-        # type: (str) -> bool
-        """Check if a path is excluded.
-
-        :param str path:
-            Path to check against the exclude patterns.
-        :returns:
-            True if there are exclude patterns and the path matches,
-            otherwise False.
-        :rtype:
-            bool
-        """
-        if path == "-":
-            if self.options.stdin_display_name == "stdin":
-                return False
-            path = self.options.stdin_display_name
-
-        return utils.matches_filename(
-            path,
-            patterns=self.options.exclude,
-            log_message='"%(path)s" has %(whether)sbeen excluded',
-            logger=LOG,
-        )
-
-    def make_checkers(self, paths=None):
-        # type: (Optional[List[str]]) -> None
-        """Create checkers for each file."""
-        if paths is None:
-            paths = self.arguments
-
-        if not paths:
-            paths = ["."]
-
-        filename_patterns = self.options.filename
-        running_from_vcs = self.options._running_from_vcs
-        running_from_diff = self.options.diff
-
-        # NOTE(sigmavirus24): Yes this is a little unsightly, but it's our
-        # best solution right now.
-        def should_create_file_checker(filename, argument):
-            """Determine if we should create a file checker."""
-            matches_filename_patterns = utils.fnmatch(
-                filename, filename_patterns
-            )
-            is_stdin = filename == "-"
-            # NOTE(sigmavirus24): If a user explicitly specifies something,
-            # e.g, ``flake8 bin/script`` then we should run Flake8 against
-            # that. Since should_create_file_checker looks to see if the
-            # filename patterns match the filename, we want to skip that in
-            # the event that the argument and the filename are identical.
-            # If it was specified explicitly, the user intended for it to be
-            # checked.
-            explicitly_provided = (
-                not running_from_vcs
-                and not running_from_diff
-                and (argument == filename)
-            )
-            return (
-                explicitly_provided or matches_filename_patterns
-            ) or is_stdin
-
-        checks = self.checks.to_dictionary()
-        checkers = (
-            FileChecker(filename, checks, self.options)
-            for argument in paths
-            for filename in utils.filenames_from(
-                argument, self.is_path_excluded
-            )
-            if should_create_file_checker(filename, argument)
-        )
-        self.checkers = [
-            checker for checker in checkers if checker.should_process
-        ]
-        LOG.info("Checking %d files", len(self.checkers))
-
-    def report(self):
-        # type: () -> Tuple[int, int]
+    def report(self) -> tuple[int, int]:
         """Report all of the errors found in the managed file checkers.
 
         This iterates over each of the checkers and reports the errors sorted
@@ -252,48 +179,28 @@ class Manager(object):
 
         :returns:
             A tuple of the total results found and the results reported.
-        :rtype:
-            tuple(int, int)
         """
         results_reported = results_found = 0
-        for checker in self.checkers:
-            results = sorted(
-                checker.results, key=lambda tup: (tup[1], tup[2])
-            )
-            filename = checker.display_name
+        self.results.sort(key=operator.itemgetter(0))
+        for filename, results, _ in self.results:
+            results.sort(key=operator.itemgetter(1, 2))
             with self.style_guide.processing_file(filename):
                 results_reported += self._handle_results(filename, results)
             results_found += len(results)
         return (results_found, results_reported)
 
-    def run_parallel(self):
+    def run_parallel(self) -> None:
         """Run the checkers in parallel."""
-        # fmt: off
-        final_results = collections.defaultdict(list)  # type: Dict[str, List[Tuple[str, int, int, str, Optional[str]]]]  # noqa: E501
-        final_statistics = collections.defaultdict(dict)  # type: Dict[str, Dict[str, None]]  # noqa: E501
-        # fmt: on
+        with _mp_prefork(self.plugins, self.options):
+            pool = _try_initialize_processpool(self.jobs, self.argv)
 
-        try:
-            pool = multiprocessing.Pool(self.jobs, _pool_init)
-        except OSError as oserr:
-            if oserr.errno not in SERIAL_RETRY_ERRNOS:
-                raise
+        if pool is None:
             self.run_serial()
             return
 
         pool_closed = False
         try:
-            pool_map = pool.imap_unordered(
-                _run_checks,
-                self.checkers,
-                chunksize=calculate_pool_chunksize(
-                    len(self.checkers), self.jobs
-                ),
-            )
-            for ret in pool_map:
-                filename, results, statistics = ret
-                final_results[filename] = results
-                final_statistics[filename] = statistics
+            self.results = list(pool.imap_unordered(_mp_run, self.filenames))
             pool.close()
             pool.join()
             pool_closed = True
@@ -302,30 +209,28 @@ class Manager(object):
                 pool.terminate()
                 pool.join()
 
-        for checker in self.checkers:
-            filename = checker.display_name
-            checker.results = sorted(
-                final_results[filename], key=lambda tup: (tup[2], tup[2])
-            )
-            checker.statistics = final_statistics[filename]
-
-    def run_serial(self):
+    def run_serial(self) -> None:
         """Run the checkers in serial."""
-        for checker in self.checkers:
-            checker.run_checks()
+        self.results = [
+            FileChecker(
+                filename=filename,
+                plugins=self.plugins,
+                options=self.options,
+            ).run_checks()
+            for filename in self.filenames
+        ]
 
-    def run(self):
+    def run(self) -> None:
         """Run all the checkers.
 
         This will intelligently decide whether to run the checks in parallel
         or whether to run them in serial.
 
         If running the checks in parallel causes a problem (e.g.,
-        https://gitlab.com/pycqa/flake8/issues/74) this also implements
-        fallback to serial processing.
+        :issue:`117`) this also implements fallback to serial processing.
         """
         try:
-            if self.jobs > 1 and len(self.checkers) > 1:
+            if self.jobs > 1 and len(self.filenames) > 1:
                 self.run_parallel()
             else:
                 self.run_serial()
@@ -333,44 +238,43 @@ class Manager(object):
             LOG.warning("Flake8 was interrupted by the user")
             raise exceptions.EarlyQuit("Early quit while running checks")
 
-    def start(self, paths=None):
+    def start(self) -> None:
         """Start checking files.
 
-        :param list paths:
+        :param paths:
             Path names to check. This is passed directly to
             :meth:`~Manager.make_checkers`.
         """
         LOG.info("Making checkers")
-        self.make_checkers(paths)
+        self.filenames = tuple(
+            expand_paths(
+                paths=self.options.filenames,
+                stdin_display_name=self.options.stdin_display_name,
+                filename_patterns=self.options.filename,
+                exclude=self.exclude,
+            )
+        )
 
-    def stop(self):
+    def stop(self) -> None:
         """Stop checking files."""
         self._process_statistics()
 
 
-class FileChecker(object):
+class FileChecker:
     """Manage running checks for a file and aggregate the results."""
 
-    def __init__(self, filename, checks, options):
-        """Initialize our file checker.
-
-        :param str filename:
-            Name of the file to check.
-        :param checks:
-            The plugins registered to check the file.
-        :type checks:
-            dict
-        :param options:
-            Parsed option values from config and command-line.
-        :type options:
-            optparse.Values
-        """
+    def __init__(
+        self,
+        *,
+        filename: str,
+        plugins: Checkers,
+        options: argparse.Namespace,
+    ) -> None:
+        """Initialize our file checker."""
         self.options = options
         self.filename = filename
-        self.checks = checks
-        # fmt: off
-        self.results = []  # type: List[Tuple[str, int, int, str, Optional[str]]]  # noqa: E501
-        # fmt: on
+        self.plugins = plugins
+        self.results: Results = []
         self.statistics = {
             "tokens": 0,
             "logical lines": 0,
@@ -384,83 +288,109 @@ class FileChecker(object):
             self.should_process = not self.processor.should_ignore_file()
             self.statistics["physical lines"] = len(self.processor.lines)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         """Provide helpful debugging representation."""
-        return "FileChecker for {}".format(self.filename)
+        return f"FileChecker for {self.filename}"
 
-    def _make_processor(self):
-        # type: () -> Optional[processor.FileProcessor]
+    def _make_processor(self) -> processor.FileProcessor | None:
         try:
             return processor.FileProcessor(self.filename, self.options)
-        except IOError as e:
+        except OSError as e:
             # If we can not read the file due to an IOError (e.g., the file
             # does not exist or we do not have the permissions to open it)
             # then we need to format that exception for the user.
             # NOTE(sigmavirus24): Historically, pep8 has always reported this
             # as an E902. We probably *want* a better error code for this
             # going forward.
-            message = "{0}: {1}".format(type(e).__name__, e)
-            self.report("E902", 0, 0, message)
+            self.report("E902", 0, 0, f"{type(e).__name__}: {e}")
             return None
 
-    def report(self, error_code, line_number, column, text, line=None):
-        # type: (str, int, int, str, Optional[str]) -> str
+    def report(
+        self,
+        error_code: str | None,
+        line_number: int,
+        column: int,
+        text: str,
+    ) -> str:
         """Report an error by storing it in the results list."""
         if error_code is None:
             error_code, text = text.split(" ", 1)
 
-        physical_line = line
         # If we're recovering from a problem in _make_processor, we will not
         # have this attribute.
-        if not physical_line and getattr(self, "processor", None):
-            physical_line = self.processor.line_for(line_number)
+        if hasattr(self, "processor") and self.processor is not None:
+            line = self.processor.noqa_line_for(line_number)
+        else:
+            line = None
 
-        error = (error_code, line_number, column, text, physical_line)
-        self.results.append(error)
+        self.results.append((error_code, line_number, column, text, line))
         return error_code
 
-    def run_check(self, plugin, **arguments):
+    def run_check(self, plugin: LoadedPlugin, **arguments: Any) -> Any:
         """Run the check in a single plugin."""
-        LOG.debug("Running %r with %r", plugin, arguments)
+        assert self.processor is not None, self.filename
         try:
-            self.processor.keyword_arguments_for(
-                plugin["parameters"], arguments
+            params = self.processor.keyword_arguments_for(
+                plugin.parameters, arguments
             )
         except AttributeError as ae:
-            LOG.error("Plugin requested unknown parameters.")
             raise exceptions.PluginRequestedUnknownParameters(
-                plugin=plugin, exception=ae
+                plugin_name=plugin.display_name, exception=ae
             )
         try:
-            return plugin["plugin"](**arguments)
+            return plugin.obj(**arguments, **params)
         except Exception as all_exc:
             LOG.critical(
                 "Plugin %s raised an unexpected exception",
-                plugin["name"],
+                plugin.display_name,
                 exc_info=True,
             )
             raise exceptions.PluginExecutionFailed(
-                plugin=plugin, exception=all_exc
+                filename=self.filename,
+                plugin_name=plugin.display_name,
+                exception=all_exc,
             )
 
     @staticmethod
-    def _extract_syntax_information(exception):
-        token = ()
-        if len(exception.args) > 1:
+    def _extract_syntax_information(exception: Exception) -> tuple[int, int]:
+        if (
+            len(exception.args) > 1
+            and exception.args[1]
+            and len(exception.args[1]) > 2
+        ):
             token = exception.args[1]
-            if token and len(token) > 2:
-                row, column = token[1:3]
+            row, column = token[1:3]
+        elif (
+            isinstance(exception, tokenize.TokenError)
+            and len(exception.args) == 2
+            and len(exception.args[1]) == 2
+        ):
+            token = ()
+            row, column = exception.args[1]
         else:
+            token = ()
             row, column = (1, 0)
 
-        if column > 0 and token and isinstance(exception, SyntaxError):
+        if (
+            column > 0
+            and token
+            and isinstance(exception, SyntaxError)
+            and len(token) == 4  # Python 3.9 or earlier
+        ):
             # NOTE(sigmavirus24): SyntaxErrors report 1-indexed column
             # numbers. We need to decrement the column number by 1 at
             # least.
             column_offset = 1
             row_offset = 0
-            # See also: https://gitlab.com/pycqa/flake8/issues/237
-            physical_line = token[-1]
+            # See also: https://github.com/pycqa/flake8/issues/169,
+            # https://github.com/PyCQA/flake8/issues/1372
+            # On Python 3.9 and earlier, token will be a 4-item tuple with the
+            # last item being the string. Starting with 3.10, they added to
+            # the tuple so now instead of it ending with the code that failed
+            # to parse, it ends with the end of the section of code that
+            # failed to parse. Luckily the absolute position in the tuple is
+            # stable across versions so we can use that here
+            physical_line = token[3]
 
             # NOTE(sigmavirus24): Not all "tokens" have a string as the last
             # argument. In this event, let's skip trying to find the correct
@@ -469,7 +399,7 @@ class FileChecker(object):
                 # NOTE(sigmavirus24): SyntaxErrors also don't exactly have a
                 # "physical" line so much as what was accumulated by the point
                 # tokenizing failed.
-                # See also: https://gitlab.com/pycqa/flake8/issues/237
+                # See also: https://github.com/pycqa/flake8/issues/169
                 lines = physical_line.rstrip("\n").split("\n")
                 row_offset = len(lines) - 1
                 logical_line = lines[0]
@@ -480,18 +410,12 @@ class FileChecker(object):
             column -= column_offset
         return row, column
 
-    def run_ast_checks(self):
+    def run_ast_checks(self) -> None:
         """Run all checks expecting an abstract syntax tree."""
-        try:
-            ast = self.processor.build_ast()
-        except (ValueError, SyntaxError, TypeError) as e:
-            row, column = self._extract_syntax_information(e)
-            self.report(
-                "E999", row, column, "%s: %s" % (type(e).__name__, e.args[0])
-            )
-            return
+        assert self.processor is not None, self.filename
+        ast = self.processor.build_ast()
 
-        for plugin in self.checks["ast_plugins"]:
+        for plugin in self.plugins.tree:
             checker = self.run_check(plugin, tree=ast)
             # If the plugin uses a class, call the run method of it, otherwise
             # the call should return something iterable itself
@@ -499,7 +423,7 @@ class FileChecker(object):
                 runner = checker.run()
             except AttributeError:
                 runner = checker
-            for (line_number, offset, text, check) in runner:
+            for (line_number, offset, text, _) in runner:
                 self.report(
                     error_code=None,
                     line_number=line_number,
@@ -507,8 +431,9 @@ class FileChecker(object):
                     text=text,
                 )
 
-    def run_logical_checks(self):
+    def run_logical_checks(self) -> None:
         """Run all checks expecting a logical line."""
+        assert self.processor is not None
         comments, logical_line, mapping = self.processor.build_logical_line()
         if not mapping:
             return
@@ -516,12 +441,13 @@ class FileChecker(object):
 
         LOG.debug('Logical line: "%s"', logical_line.rstrip())
 
-        for plugin in self.checks["logical_line_plugins"]:
+        for plugin in self.plugins.logical_line:
             self.processor.update_checker_state_for(plugin)
             results = self.run_check(plugin, logical_line=logical_line) or ()
             for offset, text in results:
-                offset = find_offset(offset, mapping)
-                line_number, column_offset = offset
+                line_number, column_offset = find_offset(offset, mapping)
+                if line_number == column_offset == 0:
+                    LOG.warning("position of error out of bounds: %s", plugin)
                 self.report(
                     error_code=None,
                     line_number=line_number,
@@ -531,12 +457,13 @@ class FileChecker(object):
 
         self.processor.next_logical_line()
 
-    def run_physical_checks(self, physical_line, override_error_line=None):
+    def run_physical_checks(self, physical_line: str) -> None:
         """Run all checks for a given physical line.
 
         A single physical check may return multiple errors.
         """
-        for plugin in self.checks["physical_line_plugins"]:
+        assert self.processor is not None
+        for plugin in self.plugins.physical_line:
             self.processor.update_checker_state_for(plugin)
             result = self.run_check(plugin, physical_line=physical_line)
 
@@ -554,62 +481,61 @@ class FileChecker(object):
 
                 for result_single in result:
                     column_offset, text = result_single
-                    error_code = self.report(
+                    self.report(
                         error_code=None,
                         line_number=self.processor.line_number,
                         column=column_offset,
                         text=text,
-                        line=(override_error_line or physical_line),
-                    )
-                    self.processor.check_physical_error(
-                        error_code, physical_line
                     )
 
-    def process_tokens(self):
+    def process_tokens(self) -> None:
         """Process tokens and trigger checks.
 
-        This can raise a :class:`flake8.exceptions.InvalidSyntax` exception.
         Instead of using this directly, you should use
         :meth:`flake8.checker.FileChecker.run_checks`.
         """
+        assert self.processor is not None
         parens = 0
         statistics = self.statistics
         file_processor = self.processor
+        prev_physical = ""
         for token in file_processor.generate_tokens():
             statistics["tokens"] += 1
-            self.check_physical_eol(token)
+            self.check_physical_eol(token, prev_physical)
             token_type, text = token[0:2]
-            processor.log_token(LOG, token)
             if token_type == tokenize.OP:
                 parens = processor.count_parentheses(parens, text)
             elif parens == 0:
                 if processor.token_is_newline(token):
                     self.handle_newline(token_type)
+            prev_physical = token[4]
 
         if file_processor.tokens:
             # If any tokens are left over, process them
             self.run_physical_checks(file_processor.lines[-1])
             self.run_logical_checks()
 
-    def run_checks(self):
+    def run_checks(self) -> tuple[str, Results, dict[str, int]]:
         """Run checks against the file."""
+        if self.processor is None or not self.should_process:
+            return self.display_name, self.results, self.statistics
+
         try:
-            self.process_tokens()
             self.run_ast_checks()
-        except exceptions.InvalidSyntax as exc:
-            self.report(
-                exc.error_code,
-                exc.line_number,
-                exc.column_number,
-                exc.error_message,
-            )
+            self.process_tokens()
+        except (SyntaxError, tokenize.TokenError) as e:
+            code = "E902" if isinstance(e, tokenize.TokenError) else "E999"
+            row, column = self._extract_syntax_information(e)
+            self.report(code, row, column, f"{type(e).__name__}: {e.args[0]}")
+            return self.display_name, self.results, self.statistics
 
         logical_lines = self.processor.statistics["logical lines"]
         self.statistics["logical lines"] = logical_lines
-        return self.filename, self.results, self.statistics
+        return self.display_name, self.results, self.statistics
 
-    def handle_newline(self, token_type):
+    def handle_newline(self, token_type: int) -> None:
         """Handle the logic when encountering a newline token."""
+        assert self.processor is not None
         if token_type == tokenize.NEWLINE:
             self.run_logical_checks()
             self.processor.reset_blank_before()
@@ -620,11 +546,20 @@ class FileChecker(object):
         else:
             self.run_logical_checks()
 
-    def check_physical_eol(self, token):
+    def check_physical_eol(
+        self, token: tokenize.TokenInfo, prev_physical: str
+    ) -> None:
         """Run physical checks if and only if it is at the end of the line."""
+        assert self.processor is not None
+        # a newline token ends a single physical line.
         if processor.is_eol_token(token):
-            # Obviously, a newline token ends a single physical line.
-            self.run_physical_checks(token[4])
+            # if the file does not end with a newline, the NEWLINE
+            # token is inserted by the parser, but it does not contain
+            # the previous physical line in `token[4]`
+            if token[4] == "":
+                self.run_physical_checks(prev_physical)
+            else:
+                self.run_physical_checks(token[4])
         elif processor.is_multiline_string(token):
             # Less obviously, a string that contains newlines is a
             # multiline string, either triple-quoted or with internal
@@ -640,38 +575,38 @@ class FileChecker(object):
             line_no = token[2][0]
             with self.processor.inside_multiline(line_number=line_no):
                 for line in self.processor.split_line(token):
-                    self.run_physical_checks(
-                        line + "\n", override_error_line=token[4]
-                    )
+                    self.run_physical_checks(line)
 
 
-def _pool_init():
-    """Ensure correct signaling of ^C using multiprocessing.Pool."""
-    signal.signal(signal.SIGINT, signal.SIG_IGN)
+def _try_initialize_processpool(
+    job_count: int,
+    argv: Sequence[str],
+) -> multiprocessing.pool.Pool | None:
+    """Return a new process pool instance if we are able to create one."""
+    try:
+        return multiprocessing.Pool(job_count, _mp_init, initargs=(argv,))
+    except OSError as err:
+        if err.errno not in SERIAL_RETRY_ERRNOS:
+            raise
+    except ImportError:
+        pass
+
+    return None
 
 
-def calculate_pool_chunksize(num_checkers, num_jobs):
-    """Determine the chunksize for the multiprocessing Pool.
-
-    - For chunksize, see: https://docs.python.org/3/library/multiprocessing.html#multiprocessing.pool.Pool.imap  # noqa
-    - This formula, while not perfect, aims to give each worker two batches of
-      work.
-    - See: https://gitlab.com/pycqa/flake8/merge_requests/156#note_18878876
-    - See: https://gitlab.com/pycqa/flake8/issues/265
-    """
-    return max(num_checkers // (num_jobs * 2), 1)
-
-
-def _run_checks(checker):
-    return checker.run_checks()
-
-
-def find_offset(offset, mapping):
+def find_offset(
+    offset: int, mapping: processor._LogicalMapping
+) -> tuple[int, int]:
     """Find the offset tuple for a single offset."""
     if isinstance(offset, tuple):
         return offset
 
-    for token_offset, position in mapping:
+    for token in mapping:
+        token_offset = token[0]
         if offset <= token_offset:
+            position = token[1]
             break
+    else:
+        position = (0, 0)
+        offset = token_offset = 0
     return (position[0], position[1] + offset - token_offset)
