@@ -1,4 +1,6 @@
 """Constants that define defaults."""
+from __future__ import annotations
+
 import re
 
 EXCLUDE = (
@@ -9,14 +11,14 @@ EXCLUDE = (
     ".git",
     "__pycache__",
     ".tox",
+    ".nox",
     ".eggs",
     "*.egg",
 )
 IGNORE = ("E121", "E123", "E126", "E226", "E24", "E704", "W503", "W504")
 SELECT = ("E", "F", "W", "C90")
 MAX_LINE_LENGTH = 79
-
-TRUTHY_VALUES = {"true", "1", "t"}
+INDENT_SIZE = 4
 
 # Other constants
 WHITESPACE = frozenset(" \t")
@@ -41,3 +43,5 @@ NOQA_INLINE_REGEXP = re.compile(
 )
 
 NOQA_FILE = re.compile(r"\s*# flake8[:=]\s*noqa", re.I)
+
+VALID_CODE_PREFIX = re.compile("^[A-Z]{1,3}[0-9]{0,3}$", re.ASCII)
