@@ -1,10 +1,16 @@
 """Plugin built-in to Flake8 to treat pyflakes as a plugin."""
+from __future__ import annotations
+
+import argparse
+import ast
 import os
-from typing import List
+from typing import Any
+from typing import Generator
 
 import pyflakes.checker
 
 from flake8 import utils
+from flake8.options.manager import OptionManager
 
 FLAKE8_PYFLAKES_CODES = {
     "UnusedImport": "F401",
@@ -41,19 +47,17 @@ FLAKE8_PYFLAKES_CODES = {
     "ContinueOutsideLoop": "F702",
     "ContinueInFinally": "F703",
     "YieldOutsideFunction": "F704",
-    "ReturnWithArgsInsideGenerator": "F705",
     "ReturnOutsideFunction": "F706",
     "DefaultExceptNotLast": "F707",
     "DoctestSyntaxError": "F721",
     "ForwardAnnotationSyntaxError": "F722",
-    "CommentAnnotationSyntaxError": "F723",
     "RedefinedWhileUnused": "F811",
-    "RedefinedInListComp": "F812",
     "UndefinedName": "F821",
     "UndefinedExport": "F822",
     "UndefinedLocal": "F823",
     "DuplicateArgument": "F831",
     "UnusedVariable": "F841",
+    "UnusedAnnotation": "F842",
     "RaiseNotImplemented": "F901",
 }
 
@@ -61,13 +65,11 @@ FLAKE8_PYFLAKES_CODES = {
 class FlakesChecker(pyflakes.checker.Checker):
     """Subclass the Pyflakes checker to conform with the flake8 API."""
 
-    name = "pyflakes"
-    version = pyflakes.__version__
     with_doctest = False
-    include_in_doctest: List[str] = []
-    exclude_from_doctest: List[str] = []
+    include_in_doctest: list[str] = []
+    exclude_from_doctest: list[str] = []
 
-    def __init__(self, tree, file_tokens, filename):
+    def __init__(self, tree: ast.AST, filename: str) -> None:
         """Initialize the PyFlakes plugin with an AST tree and filename."""
         filename = utils.normalize_path(filename)
         with_doctest = self.with_doctest
@@ -82,24 +84,19 @@ class FlakesChecker(pyflakes.checker.Checker):
         for exclude in self.exclude_from_doctest:
             if exclude != "" and filename.startswith(exclude):
                 with_doctest = False
-                overlaped_by = [
+                overlapped_by = [
                     include
                     for include in included_by
                     if include.startswith(exclude)
                 ]
 
-                if overlaped_by:
+                if overlapped_by:
                     with_doctest = True
 
-        super().__init__(
-            tree,
-            filename=filename,
-            withDoctest=with_doctest,
-            file_tokens=file_tokens,
-        )
+        super().__init__(tree, filename=filename, withDoctest=with_doctest)
 
     @classmethod
-    def add_options(cls, parser):
+    def add_options(cls, parser: OptionManager) -> None:
         """Register options for PyFlakes on the Flake8 OptionManager."""
         parser.add_option(
             "--builtins",
@@ -134,7 +131,7 @@ class FlakesChecker(pyflakes.checker.Checker):
         )
 
     @classmethod
-    def parse_options(cls, options):
+    def parse_options(cls, options: argparse.Namespace) -> None:
         """Parse option values from Flake8's OptionManager."""
         if options.builtins:
             cls.builtIns = cls.builtIns.union(options.builtins)
@@ -171,7 +168,7 @@ class FlakesChecker(pyflakes.checker.Checker):
                 f"both for doctesting."
             )
 
-    def run(self):
+    def run(self) -> Generator[tuple[int, int, str, type[Any]], None, None]:
         """Run the plugin."""
         for message in self.messages:
             col = getattr(message, "col", 0)
